<?php

namespace JimsGroup\Apis;

use JimsGroup\Datatypes\FMSAuthResponseType;
use JimsGroup\Utilities\HttpRequest;

/**
 *
 */
class FMSAuthentication extends AbstractApi
{
    /**
     * @var string
     */
    protected $mode = 'production';

    /**
     * Who requests the token
     * @var string
     */
    protected $client_id = 'fms_user';

    /**
     * What credentials to use to get the token
     * @var string
     */
    protected $grant_type = 'password';


    /**
     * @var FMSAuthResponseType|null
     */
    protected $user = null;


    /**
     * @var string
     */
    protected $token = '';

    /**
     * @var string
     */
    protected $scope = 'openid profile roles jimsApi session';

    protected $api_urls = [
        'production' => 'https://id.jims.net',
        'development' => 'https://id-test.cloud.jims.net',
    ];

    protected $paths = [
        'identity' => '/identity/connect/token'
    ];

    public function __construct($token, $mode = 'production')
    {
        $this->token = $token;
        parent::__construct($mode);
    }

    /**
     * @param $username
     * @param $password
     * @return boolean
     */
    public function connect($username, $password)
    {
        $response = $this->getToken($username, $password);
        if($response){
            $this->user = new FMSAuthResponseType($response['payload']);
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return FMSAuthResponseType|null
     */
    public function getUser(){
        return $this->user;
    }


    /**
     * @param $username
     * @param $password
     * @return mixed|null
     */
    protected function getToken($username, $password)
    {
        $http = new HttpRequest();
        $url = $this->getEndpointUrl('identity');
        $data = [
            'client_id' => $this->client_id,
            'grant_type' => $this->grant_type,
            'scope' => $this->scope,
            'username' => $username,
            'password' => $password,
        ];
        $http->addHeader ( 'Authorization: Basic ' . $this->token);
        $response = $http->post($url, $data);

        //if token is not empty, then we are connected
        if(strlen($response) === 0 ) return null;

        $data = json_decode($response, true);
        if( !empty($data) && isset($data['access_token'])){
            return HttpRequest::parseJWT ( $data['access_token'] );
        }

        return null;
    }
}